#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <memory.h>
#include <fcntl.h>

#include "fmudrv2.h"
#include "..\common.h"
#include "..\extern.h"
#include "..\gmpv4sp.h"

char	*FMUDRV2_fmneiro = "FMNEIRO.WAV";
char	*FMUDRV2_mdneiro = "MDNEIRO.WAV";
char	*FMUDRV2_drum = "DRUM.DAT";
char	*FMUDRV2_se = "SE.DAT";

int FMUDRV2_check( void )
{
	char	id[16];

	memset( id, 0, sizeof( id ));

	_readvect( 0x40, 0, 0x7D, id );

	if( memcmp( id, "FMU", 3 ) != 0 )
	{
		if( memcmp( id, "MMU", 3 ) != 0 )
		{
			printf( MES[MES_DRVNOTFOUND], DRIVER_name );
			return 1;
		}
	}

	printf( INFO[INFO_DRIVER], DRIVER_name );

	return 0;
}

int FMUDRV2_load( char *filename )
{
	struct find_t	ft;

	/* Init */
	r.h.ah = 0x0F;
	int86x( 0x40, &r, &ro, &sr );

	/* Music(page0) */
	GMPV4SP_address( &seg, &off, 0 );
	if( GMPV4SP_load( filename, 0, 0, 1 ) == -1 ) return 1;
	r.h.ah = 0x01;
	r.x.dx = seg;
	int86x( 0x40, &r, &ro, &sr );

	if( GMPV4SP_size( 1 ) == -1 ) return 1;

	GMPV4SP_address( &seg, &off, 1 );
	GMPV4SP_clear( 1 );

	/* Tone FM(page1) */
	if( _dos_findfirst( FMUDRV2_fmneiro, _A_NORMAL, &ft ) == 0 )
	{
		if( GMPV4SP_load( ft.name, 0, 1, 0 ) == -1 ) return 1;

		r.h.ah = 0x02;
		r.x.dx = seg;
		int86x( 0x40, &r, &ro, &sr );
	}

	/* Tone MIDI(page1) */
	if( _dos_findfirst( FMUDRV2_mdneiro, _A_NORMAL, &ft ) == 0 )
	{
		if( GMPV4SP_load( ft.name, 0, 1, 0 ) == -1 ) return 1;

		r.h.ah = 0x02;
		r.x.dx = seg;
		int86x( 0x40, &r, &ro, &sr );
	}

	/* Tone SSGDrum(page1) */
	if( _dos_findfirst( FMUDRV2_drum, _A_NORMAL, &ft ) == 0 )
	{
		if( GMPV4SP_load( ft.name, 0x07000000, 1, 0 ) == -1 ) return 1;

		r.h.ah = 0x03;
		r.x.dx = seg + 0x700;
		int86x( 0x40, &r, &ro, &sr );
	}

	/* Tone SE(page1) */
	if( _dos_findfirst( FMUDRV2_se, _A_NORMAL, &ft ) == 0 )
	{
		if( GMPV4SP_load( ft.name, 0x08000000, 1, 0 ) == -1 ) return 1;

		r.h.ah = 0x04;
		r.x.dx = seg + 0x800;
		int86x( 0x40, &r, &ro, &sr );
	}

	/* Send */
	r.h.ah = 0x00;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}

int FMUDRV2_play( char *opt1, char *opt2 )
{
	if( strcmpi( opt1, OPT[OPT_LOOP] ) == 0 ) r.h.al = 0xFF;
	else r.h.al = 0x00;

	r.h.ah = 0x05;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}

int FMUDRV2_stop( void )
{
	r.h.ah = 0x06;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}

int FMUDRV2_fade( void )
{
	r.h.ah = 0x07;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}
