#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <memory.h>
#include <fcntl.h>

#include "mako.h"
#include "..\common.h"
#include "..\extern.h"
#include "..\gmpv4sp.h"

char	MAKO_filename[13];
char	*MAKO_dri_name = "DRI";
int		MAKO_dri_exist;
int		MAKO_mode;

int MAKO_check( void )
{
	char	id[16];

	MAKO_dri_exist = 0;

	/* Mako */
	memset( id, 0, sizeof( id ));
	_readvect( 0x40, 0, -2, id );

	if( memcmp( id, "MK", 2 ) != 0 )
	{
		printf( MES[MES_DRVNOTFOUND], DRIVER_name );
		return 1;
	}

	printf( INFO[INFO_DRIVER], DRIVER_name );

	/* Dri */
	memset( id, 0, sizeof( id ));
	_readvect( 0x41, 0, -2, id );

	if( memcmp( id, "DR", 2 ) == 0 )
	{
		MAKO_dri_exist = 1;
		printf( INFO[INFO_DRIVER], MAKO_dri_name );
	}

	return 0;
}

int MAKO_load( char *filename )
{
	/* Create filename */
	sprintf( MAKO_filename, "%s", filename );

	return 0;
}

int MAKO_play( char *opt1, char *opt2 )
{
	if( strcmpi( opt1, OPT[OPT_DRI] ) == 0 ) MAKO_mode = 1;
	if( strcmpi( opt1, OPT[OPT_DIRECT] ) == 0 ) MAKO_mode = 0;

	if( strcmpi( opt2, OPT[OPT_SELECTOR] ) == 0 )
	{
		MAKO_selector();
		return 0;
	}

	if( strcmpi( opt1, OPT[OPT_DRI] ) == 0 )
	{
		MAKO_play_dri( opt1, opt2 );
		return 0;
	}

	if( strcmpi( opt1, OPT[OPT_DIRECT] ) == 0 )
	{
		MAKO_play_direct( opt1, opt2 );
		return 0;
	}

	return 0;
}

int	MAKO_play_dri( char *opt1, char *opt2 )
{
	long	filesize;
	char	filename[16];

	sprintf( filename, "%s:AMUS.DAT", MAKO_filename );

	if( MAKO_dri_exist == 0 )
	{
		printf( MES[MES_DRVNOTFOUND], MAKO_dri_name );
		return 1;
	}

	filesize = _filesize( filename );
	if( filesize == -1 ) return 1;

	GMPV4SP_address( &seg, &off, 0 );

	/*
		Music Load(DRI) filename : DS:DX
		                buffer   : BX:BP
	*/
	r.h.ah = 0x00;
	sr.ds  = FP_SEG( filename );
	r.x.dx = FP_OFF( filename );
	r.x.bx = seg;
	r.x.bp = off;

	r.h.ch = 0x00;
	if( opt2 == NULL ) r.h.cl = 0x00;
	else r.h.cl = atoi( opt2 );

	int86x( 0x41, &r, &ro, &sr );
	if( ro.x.ax != 0 ) return 1;

	printf( INFO[INFO_LOAD], filename, filesize );
	printf( INFO[INFO_NO], r.h.cl );

	MAKO_play_common();

	return 0;
}

int	MAKO_play_direct( char *opt1, char *opt2 )
{
	int					in;
	unsigned int		readsize;
	long				filesize;
	unsigned char far	*p = NULL;
	int					no_max;
	int					no;
	char				filename[13];
	unsigned long		data_start;
	unsigned long		data_ed;

	sprintf( filename, "%s", MAKO_filename );

	no_max = 0;
	if( opt2 == NULL ) no = 0x00;
	else no = atoi( opt2 );

	filesize = _filesize( filename );
	if( filesize == -1 ) return 1;

	GMPV4SP_address( &seg, &off, 0 );

	p = MK_FP( seg, off );

	_dos_open( filename, O_RDONLY, &in );
	_dos_read( in, p, 2, &readsize );

	if(*( unsigned int far *)( p ) == 2 ) no_max = 127;
	if(*( unsigned int far *)( p ) == 3 ) no_max = 255;
	if( no > no_max )
	{
		_dos_close( in );
		return 1;
	}

	lseek( in, ( no * 2 ) + 2, SEEK_SET );
	_dos_read( in, p, 4, &readsize );
	if(
		(*( unsigned int far *)( p ) == 0 ) ||
		(*( unsigned int far *)( p + 2 ) == 0 )
	)
	{
		_dos_close( in );
		return 1;
	}

	data_start = (*( unsigned int far *)( p ) - 1 ) * 256;
	data_ed = (*( unsigned int far *)( p + 2 ) - 1 ) * 256;

	lseek( in, data_start, SEEK_SET );
	_dos_read( in, p, ( data_ed - data_start ), &readsize );
	_dos_close( in );

	printf( INFO[INFO_LOAD], filename, filesize );
	printf( INFO[INFO_NO], no );

	MAKO_play_common();

	return 0;
}

int	MAKO_play_common( void )
{
	unsigned char far	*p = NULL;

	GMPV4SP_address( &seg, &off, 0 );

	/* Music load(MAKO) : DS:SI */
	p = MK_FP( seg, off );
	if(*( unsigned int far *)( p ) > 0x100 ) r.h.ah = 0x40;	/* SSGPCM */
	else r.h.ah = 0x00;	/* Music */

	sr.ds  = seg;
	r.x.si = off;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}

int MAKO_stop( void )
{
	r.x.ax = 0x0100;
	int86x( 0x40, &r, &ro, &sr );

	return 0;
}

int MAKO_selector( void )
{
	int		no;
	char	buf[3];

	no = 0;

	for( ;; )
	{
		switch( getch())
		{
			case 0x0A:	/* DOWN */
				if( no > 0 ) no--;
				printf( INFO[INFO_NO], no );
				break;
			case 0x0B:	/* UP */
				if( no < 255 ) no++;
				printf( INFO[INFO_NO], no );
				break;
			case 0x0D:	/* ENTER */
				sprintf( buf, "%d", no );
				MAKO_stop();
				if( MAKO_mode == 0 ) MAKO_play_direct( NULL, buf );
				else MAKO_play_dri( NULL, buf );
				break;
			case 0x1B:	/* ESC */
				return 0;
			default: break;
		}
	}
}
