#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <memory.h>
#include <fcntl.h>
#include <farstr.h>

#include "gmpv4sp.h"
#include "common.h"
#include "extern.h"

char *GMPV4SP_name = "GMPV4SP";

int GMPV4SP_check( void )
{
	char			id[16];
	unsigned int	buffer_count;
	unsigned char 	i;

	memset( id, 0, sizeof( id ));
	_readvect( 0x78, 0, -8, id );

	if( memcmp( id, "gmpv4sp", 7 ) != 0 )
	{
		printf( MES[MES_DRVNOTFOUND], GMPV4SP_name );
		return 1;
	}

	printf( INFO[INFO_DRIVER], GMPV4SP_name );

	r.h.ah = 0x07;
	int86x( 0x78, &r, &ro, &sr );
	buffer_count = ro.x.ax;

	for( i = 0; i != buffer_count; i++ )
	{
		r.h.ah = 0x08;
		r.x.bx = i;
		int86x( 0x78, &r, &ro, &sr );

		printf( INFO[INFO_GMPV4SP],
			i,
			ro.x.ax,
			0,
			ro.x.cx,
			( long )( ro.x.cx * 16 )
		);
	}

	return 0;
}

int GMPV4SP_buffer_count( void )
{
	r.h.ah = 0x07;
	int86x( 0x78, &r, &ro, &sr );

	return ro.x.ax;
}

long GMPV4SP_load( char *filename, unsigned long offset, int page, char clear )
{
	unsigned short		seg;
	unsigned short		off;
	long				filesize;
	int					buffer_count;
	unsigned char far	*p = NULL;

	filesize = _filesize( filename );
	if( filesize == -1 ) return -1;

	buffer_count = GMPV4SP_buffer_count();
	if(( buffer_count == 0 ) || ( buffer_count < page ))
	{
		printf( MES[MES_PAGENOTFOUND], GMPV4SP_name, page );
		return -1;
	}

	if(( GMPV4SP_size( page ) + offset ) < filesize )
	{
		printf( MES[MES_BUFSMALL], GMPV4SP_name, page );
		return -1;
	}

	GMPV4SP_address( &seg, &off, page );
	p = ( unsigned char far * )((( seg << 16 ) + off ) + offset );

#ifdef _DEBUG
	printf( "*Load  : [%02d] %08Fp\n", page, p );
#endif

	if( clear == 1 ) GMPV4SP_clear( page );
	_far_fileload( filename, p );

	return filesize;
}

long GMPV4SP_size( int page )
{
	r.h.ah = 0x08;
	r.x.bx = page;
	int86x( 0x78, &r, &ro, &sr );

	if( ro.x.cflag == 1 )
	{
		printf( MES[MES_PAGENOTFOUND], GMPV4SP_name, page );
		return -1;
	}
	return ( long )( ro.x.cx * 16 );
}

void GMPV4SP_clear( int page )
{
	unsigned char far	*p = NULL;
	unsigned short		seg;
	unsigned short		off;
	long				size;

	size = GMPV4SP_size( page );
	GMPV4SP_address( &seg, &off, page );
	p = MK_FP( seg, off );

	far_memset( p, 0x00, size );
}

void GMPV4SP_address( unsigned short *seg, unsigned short *off, int page )
{
	r.h.ah = 0x08;
	r.x.bx = page;
	int86x( 0x78, &r, &ro, &sr );

	*seg = ro.x.ax;
	*off = 0x0000;
}
